# Copyright (c) 2011-2024 by Ivanti, Inc. All rights reserved
# LogsAndDiagnostics.sh 
# This is a shell script to save logs&diagnostics for Pulse on Mac OS/X platforms.
# This also belongs to liblogsAndDiagnostics.a (in junsSDK), which provides a thin 
# static-lib wrapper for Pulse UI to call this script in a interruptible way and it
# also provides progress updates tp Pulse UI for display purposes. 


# Initializing...

unset -f set
unset -f echo
unset -f exit
unset -f read
unset -f printf
unset -f shift
unset -f cd

if [ -e /usr/bin/echo ]; then
  ECHO=/usr/bin/echo
elif [ -e /bin/echo ]; then
  ECHO=/bin/echo
elif [ -e /usr/sbin/echo ]; then
  ECHO=/usr/sbin/echo
elif [ -e /sbin/echo ]; then
  ECHO=/sbin/echo
else
  printf "Unable to find full path for echo command . Exiting"
  exit 5
fi

full_path_command() {
    cmd="$1"
    # Check if command exists in /usr/bin
    if [ -e "/usr/bin/$cmd" ]; then
        $ECHO "/usr/bin/$cmd"
        return
    fi
    # Check if command exists in /usr/sbin
    if [ -e "/usr/sbin/$cmd" ]; then
        $ECHO "/usr/sbin/$cmd"
        return
    fi
    # Check if command exists in /bin
    if [ -e "/bin/$cmd" ]; then
        $ECHO "/bin/$cmd"
        return
    fi
    if [ -e "/sbin/$cmd" ]; then
        $ECHO "/sbin/$cmd"
        return
    fi
	$ECHO "$cmd command not found in /usr/bin, /usr/sbin ,/sbin or /bin. exiting"
    exit 4
}


RM=$(full_path_command "rm")
MKDIR=$(full_path_command "mkdir")
NETSTAT=$(full_path_command "netstat")
IFCONFIG=$(full_path_command "ifconfig")
IP=$(full_path_command "ip")
CP=$(full_path_command "cp")
ZIP=$(full_path_command "zip")
LN=$(full_path_command "ln")
TAIL=$(full_path_command "tail")
LS=$(full_path_command "ls")
GREP=$(full_path_command "grep")
HEAD=$(full_path_command "head")
DPKG=$(full_path_command "dpkg")
RPM=$(full_path_command "rpm")
JOURNALCTL=$(full_path_command "journalctl")


$ECHO 'Initializing...'
$RM -f "$1"    # delete old file


# create temporary directory
$RM -rf /tmp/pulsesecure
$MKDIR -p /tmp/pulsesecure/network

# Saving Diagnostics info...
$ECHO 'Saving Diagnostics info...'
cd /tmp/pulsesecure/network; set > set.txt
cd /tmp/pulsesecure/network; $IFCONFIG > ifconfig.txt
cd /tmp/pulsesecure/network; $NETSTAT -nra > netstat.txt
cd /tmp/pulsesecure/network; $NETSTAT -s > netstat-s.txt
cd /tmp/pulsesecure/network; $IP addr > ipaddr.txt
cd /tmp/pulsesecure/network; $IP link > iplink.txt
cd /tmp/pulsesecure/network; $CP /etc/resolv.conf resolvconf.txt
cd /tmp/pulsesecure/network; $IP route > iproute.txt
cd /tmp/pulsesecure/network; $IP -6 route > ip6route.txt
cd /tmp/pulsesecure/network; $CP  /usr/share/dbus-1/system.d/net.psecure.pulse.conf pulseconf.txt
cd /tmp/pulsesecure/network; $NETSTAT -nr > netstat-nr.txt
cd /tmp/pulsesecure/network; $IFCONFIG > ifconfig.txt

cd /tmp/pulsesecure/network; $DPKG --list > dpkglist.txt
cd /tmp/pulsesecure/network; $RPM -qa > rpmlist.txt
cd /tmp/pulsesecure; $ZIP "$1" network/*

# Saving Configuration files...
$ECHO 'Saving Configuration files...'
cd /opt/pulsesecure/lib/JUNS/; $ZIP "$1" access.ini
#Enable below two when we support respective features
#cd /; zip "$1" tmp/pulse*    #capture manifest file generated by iveCM for component upgrade checking
#cd <>; zip "$1" pulse.pac     #capture pulse.pac for proxy

# Saving Database files...
$ECHO 'Saving Database files...'
cd /var/lib/pulsesecure; $ZIP "$1" pulse/connstore.*

# Saving DebugLog and FirewallKextLog files...
$ECHO 'Saving DebugLog files...'
$MKDIR -p /tmp/pulsesecure/logs
$LN -sf /var/log/pulsesecure/logging/debuglog.log /tmp/pulsesecure/logs/debuglog.log
$LN -sf /var/log/pulsesecure/logging/debuglog.log.old /tmp/pulsesecure/logs/debuglog.log.old
$LN -sf /var/log/pulsesecure/logging/PulseClient.log /tmp/pulsesecure/logs/PulseClient.log
$LN -sf /var/log/pulsesecure/logging/PulseClient.log.old /tmp/pulsesecure/logs/PulseClient.log.old
$LN -sf /var/log/pulsesecure/logging/dsOpenSSL.log /tmp/pulsesecure/logs/dsOpenSSL.log
$LN -sf /var/log/pulsesecure/logging/dsOpenSSL.log.old /tmp/pulsesecure/logs/dsOpenSSL.log.old
CEF_LOG_NAME="cefinstaller.log"
CEF_LOG="/var/log/pulsesecure/logging/$CEF_LOG_NAME"
if [ -f "$CEF_LOG" ]; then
    $LN -sf "$CEF_LOG" /tmp/pulsesecure/logs/$CEF_LOG_NAME
fi
CEF_TMP_LOG="/tmp/$CEF_LOG_NAME"
if [ -f $CEF_TMP_LOG ]; then
    $LN -sf $CEF_TMP_LOG /tmp/pulsesecure/logs/tmp_$CEF_LOG_NAME
fi
CEF_DBG_LOG="$HOME/.pulsesecure/cef_browser.log"
CEF_STD_ERR_LOG="$HOME/.pulsesecure/cef_browser_stderr.log"
if [ -f $CEF_DBG_LOG ]; then
    $LN -sf $CEF_DBG_LOG /tmp/pulsesecure/logs/cef_browser.log
fi
if [ -f $CEF_STD_ERR_LOG ]; then
    $LN -sf $CEF_STD_ERR_LOG /tmp/pulsesecure/logs/cef_browser_stderr.log
fi
SYSLOG=/var/log/syslog
if [ -f $SYSLOG ]; then
    $TAIL -10000 $SYSLOG > /tmp/pulsesecure/logs/system.log
elif [ -f $JOURNALCTL ]; then
    $JOURNALCTL | $TAIL -10000 > /tmp/pulsesecure/logs/system.log
fi

cd /tmp/pulsesecure; $ZIP -r "$1" logs

#Ubuntu crash location
CrashDumpDir=/var/crash/

#Fedora crash location
if [ ! -d "$CrashDumpDir" ]; then
    CrashDumpDir=/var/lib/systemd/coredump/
fi
#take only max of 10 files. Dont make the zip so big
CrashFiles=`$LS -t $CrashDumpDir | $GREP -i "pulseUI\|jamCommand\|cefSubprocess\|pulsesecure\|pulselauncher\|cefBrowser" | $HEAD -n10`

$MKDIR -p /tmp/pulsesecure/CrashDumps
cd /tmp/pulsesecure/

for crashFile in $CrashFiles; do
    $ECHO $CrashDumpDir$crashFile
    $CP $CrashDumpDir$crashFile CrashDumps
done

cd /tmp/pulsesecure;  $ZIP -r "$1"  CrashDumps

$RM -rf /tmp/pulsesecure

# Succeeded saving zip file: "$1"
$ECHO 'Succeeded saving zip file: '"$1"
$LS -l "$1"
