#!/bin/bash

# Copyright (c) 2021-2024 by Ivanti, Inc. All rights reserved
#set -x

if [ -e "/bin/uname" ]; then
    UNAME=/bin/uname
else
    UNAME=/usr/bin/uname
fi
PLATFORM=`$UNAME`
ARCH=`$UNAME -m`

# All disk space in MegaBytes
CEF_DISPLAY_NAME="Chromium Embedded Browser"

if [ "$PLATFORM" == "Linux" ]; then
    TMP_SPACE_NEEDED=1276
    INSTALL_SPACE_NEEDED=1063
    CEF_INSTALL_ROOT_DIR=/opt
    CEF_INSTALL_DIR=${CEF_INSTALL_ROOT_DIR}/pulsesecure/lib/cefRuntime
    CEF_LIB_NAME=libcef.so
    CEF_LIB_SHA2=fe5089b74ffd72d5bbd37f6e1466a71c4459ea53f19b21d145ce07569668b257
    CEF_URL=https://pcstp.pulsesecure.net/cef/linux/cef64.132.tar.bz2
    CEF_PACKAGE_SHA2=36c7bf557eccc683c0830d52f9f99a672425ed441e35dc74b58f667dc879259e
    CEF_CHECK_INSTALL_PATH=$CEF_INSTALL_DIR/Release/$CEF_LIB_NAME
    CEF_DOWNLOAD_FILE=cef64.132.tar.bz2
    CEF_PACKAGE_NAME=cef_binary_132.3.1+g144febe+chromium-132.0.6834.83_linux64_minimal
    CEF_RESOURCE_DIR=$CEF_PACKAGE_NAME/Resources
    CEF_RELEASE_DIR=$CEF_PACKAGE_NAME/Release
    PULSE_LOG_DIR="/var/log/pulsesecure/logging"
    TMP_DOWNLOAD_DIR="/opt/pulsesecure/tmp_cef"
else
    TMP_SPACE_NEEDED=300
    INSTALL_SPACE_NEEDED=200
    CEF_INSTALL_ROOT_DIR=/Library
    CEF_DIRNAME="Chromium Embedded Framework.framework"
    CEF_INSTALL_DIR2="${CEF_INSTALL_ROOT_DIR}/Application Support/Pulse Secure"
    CEF_INSTALL_DIR="${CEF_INSTALL_DIR2}/${CEF_DIRNAME}"
    CEF_LIB_NAME="Chromium Embedded Framework"
    TMP_DOWNLOAD_DIR="$CEF_INSTALL_DIR2/tmp_cef"

    if [ "$ARCH" == "x86_64" ]; then
        CEF_URL=https://pcstp.pulsesecure.net/cef/macOS/cef64.115.zip
        CEF_PACKAGE_SHA2=683d95cf7a2319de240ebc18cacc1f6f4eaf1ca075f05bdc803c98ca06d3440c
        CEF_LIB_SHA2=66c1149b259b907963ceae6baeb2d0bb28d315fc5c7533468549bb0d4c397bde
    else
        CEF_URL=https://pcstp.pulsesecure.net/cef/macOS/cefarm64.115.zip
        CEF_PACKAGE_SHA2=849d475a724959a997f332c4117038d760d9752d6b0a3180b3319208fc81ceae
        CEF_LIB_SHA2=28ecd9610b32f4520f2bfc3bede923dbd0dd2c8e09772747fa9706e556a49a28
    fi

    CEF_CHECK_INSTALL_PATH="$CEF_INSTALL_DIR/$CEF_LIB_NAME"
    CEF_DOWNLOAD_FILE=cef64.zip
    PULSE_LOG_DIR="/var/log/Pulse Secure/Logging"
fi

REINSTALL_CEF=false
LOG_DIR=$PULSE_LOG_DIR
TMP_DIR_PARAM=tmpDirPath
USR_TMP_DIR=""
attempting_copy=false
# find echo command manually
if [ -e /usr/bin/echo ]; then
  ECHO=/usr/bin/echo
elif [ -e /bin/echo ]; then
  ECHO=/bin/echo
else
  ECHO=/usr/sbin/echo
fi

full_path_command() {
    cmd="$1"
    
    # Check if command exists in /usr/bin
    if [ -e "/usr/bin/$cmd" ]; then
        $ECHO "/usr/bin/$cmd"
        return
    fi
    
    # Check if command exists in /usr/sbin
    if [ -e "/usr/sbin/$cmd" ]; then
        $ECHO "/usr/sbin/$cmd"
        return
    fi
    
    # Check if command exists in /bin
    if [ -e "/bin/$cmd" ]; then
        $ECHO "/bin/$cmd"
        return
    fi

    $ECHO "$cmd command not found in /usr/bin, /usr/sbin or /bin." >> "$LOG_DIR/cefinstaller.log"
}

DATE=$(full_path_command "date")
function get_time()
{
    $DATE '+%Y/%m/%d %H:%M:%S.%N'
}

function log_message()
{
    LOG_FILE="$LOG_DIR/cefinstaller.log"
    msg=$1
    $ECHO $msg
    $ECHO "$(get_time) $msg" >> "$LOG_FILE"
}

if [ "$#" -gt 3 ]; then
    log_message "Too many arguments provided. Exiting."
    exit 1
fi

# Check if curl or wget is installed or not , as one of them is needed for Downloading CEF
CURL=$(full_path_command "curl")
if ! [ -f "$CURL" ]; then
    log_message "curl is not installed. Checking if wget is installed or not..."
    #check if wget is installed or not    
    WGET=$(full_path_command "wget")
    if ! [ -f "$WGET" ]; then
     log_message "curl and wget are not installed. Please install either curl or wget to proceed with CEF installation, Exiting..."
     exit 4
    fi
fi

LS=$(full_path_command "ls")
CP=$(full_path_command "cp")
MKDIR=$(full_path_command "mkdir")
RM=$(full_path_command "rm")
CHMOD=$(full_path_command "chmod")
CHOWN=$(full_path_command "chown")
DF=$(full_path_command "df")
CUT=$(full_path_command "cut")
HEAD=$(full_path_command "head")
GREP=$(full_path_command "grep")
AWK=$(full_path_command "awk")
SHASUM=$(full_path_command "shasum")
TR=$(full_path_command "tr")
BASE64=$(full_path_command "base64")
CUT=$(full_path_command "cut")
UNZIP=$(full_path_command "unzip")
TAR=$(full_path_command "tar")
BASENAME=$(full_path_command "basename")
SCRNAME=$($BASENAME "$0")

function create_secure_dir()
{
    if [ -e /dev/urandom ]; then
        rnd_dev="/dev/urandom"
        log_message "Using /dev/urandom device."
    elif [ -e /dev/random ]; then
        rnd_dev="/dev/random"
        log_message "Using /dev/random device."
    else
        return_error 9 "No random device available."
    fi
    
    if [ "$USR_TMP_DIR" != "" ]; then
        log_message "Using User Supplied Directory for Downloading CEF Binaries"
        TMP_DIR=$USR_TMP_DIR
    else
        log_message "Generating Temp Directory for Downloading CEF Binaries"
        $MKDIR "$TMP_DOWNLOAD_DIR"
        if [ $? -ne 0 ]; then
            $ECHO "Failed to create directory tmp $TMP_DOWNLOAD_DIR"
            exit 9
        fi    
    	TMP_DIR=$TMP_DOWNLOAD_DIR/cef_$($HEAD -c 10 "$rnd_dev" | $BASE64 | $TR -dc 'a-zA-Z0-9' | $HEAD -c 10)
        $MKDIR -m 700 "$TMP_DIR"
        if [ $? -ne 0 ]; then
            $ECHO "Failed to create directory $TMP_DIR"
            exit 9
        else
            log_message "Created Secure Temporary Directory. $TMP_DIR"
            
            if $LS $TMP_DIR/* &>/dev/null
            then
               log_message "Download Directory is not empty Exiting. $TMP_DIR"
            fi
        fi
    fi
}

function printUsage()
{
    $ECHO "Utility to download and install $CEF_DISPLAY_NAME."
    $ECHO "Usage: $SCRNAME <check_installed|install|reinstall|uninstall> [OPTIONS]"
    $ECHO ""
    $ECHO "    $SCRNAME check_installed"
    $ECHO "                 Checks and informs if currently installed or not."
    $ECHO "    $SCRNAME <install|reinstall> [-$TMP_DIR_PARAM <Path>]"
    $ECHO "                 Downloads and installs the $CEF_DISPLAY_NAME."
    $ECHO "                 An optional -$TMP_DIR_PARAM can be passed, where download and"
    $ECHO "                 extraction will happen. This directory will be cleared by the"
    $ECHO "                 end of installation."
    $ECHO "                 install will install CEF only if it is not installed."
    $ECHO "                 reinstall will remove and reinstall if it is installed."
    $ECHO "    $SCRNAME uninstall"
    $ECHO "                 Uninstall CEF libraries"
    $ECHO "    Note:"
    $ECHO "        * The install can be run only with root privileges"
    $ECHO "        * $TMP_SPACE_NEEDED MB of free space is needed in the"
    $ECHO "          $TMP_DIR_PARAM. This space is used only during installation"
    $ECHO "          time and freed by the end of installation."
    $ECHO "        * $INSTALL_SPACE_NEEDED MB of free space is needed in the ${CEF_INSTALL_ROOT_DIR}"
    exit 1
}



ALREADY_INSTALLED=false
function check_installed()
{
    if [ -f "$CEF_CHECK_INSTALL_PATH" ]; then
        log_message "$CEF_DISPLAY_NAME is installed"
        validate_lib_checksum
        ALREADY_INSTALLED=true
    else
        log_message "$CEF_DISPLAY_NAME is not installed"
    fi
}

function get_root_dir()
{
    $ECHO "$1" | $CUT -d "/" -f2
}

function get_file_system()
{
    $DF "$1" | $GREP -vw Filesystem | $AWK {'print $1'}
}

function get_free_space()
{
    $DF -m "$1" | $GREP -vw Filesystem | $AWK {'print $4'}
}

function get_fs_desc()
{
    $DF -m "$1" | $GREP -vw Filesystem | $AWK {'printf "%s(%s)", $6, $1'}
}

function get_space_message()
{
    $ECHO "Not enough free disk space on $(get_fs_desc $1). Needed:$2MB Available:$3MB"
}

function check_partition_space()
{
    dirPath=$1
    freespaceneeded=$2
    freediskspace=$(get_free_space $dirPath)
    if [ $freediskspace -le $freespaceneeded ]; then
        log_message "$(get_space_message $dirPath $freespaceneeded $freediskspace)"
        exit 2
    fi
}

function check_root_privilege()
{
    if [ $EUID -ne 0 ]; then
        log_message "'$SCRNAME install|reinstall|uninstall' needs to be run with root privileges"
        exit 3
    fi
}

function check_disk_space()
{
    log_message "Checking disk space needed..."
    if [ ! -d "$TMP_DIR" ]; then
        $MKDIR -p "$TMP_DIR"
    fi
    tmpPartition=$(get_file_system "$TMP_DIR")
    cefInstallRoot=/$(get_root_dir $CEF_INSTALL_DIR)
    installPartition=$(get_file_system $cefInstallRoot)
    if [ "$tmpPartition" == "$installPartition" ]; then
        check_partition_space $cefInstallRoot $(($TMP_SPACE_NEEDED + $INSTALL_SPACE_NEEDED))
    else
        check_partition_space $cefInstallRoot $INSTALL_SPACE_NEEDED
        check_partition_space $TMP_DIR $TMP_SPACE_NEEDED
    fi
    log_message "Disk space available"
}

function return_error()
{
    exitValue=$1
    errorMessage=$2

    log_message "$errorMessage"
    if [ $exitValue -ne 0 ]; then
        cleanup
    fi
    exit $exitValue
}

function cleanup()
{
    log_message "Cleaning up..."
     $ECHO "$TMP_DIR/$CEF_PACKAGE_NAME"
    if [ -d "$TMP_DIR/$CEF_PACKAGE_NAME" ]; then
        log_message "Removing $TMP_DIR/$CEF_PACKAGE_NAME"
        $RM -rf "$TMP_DIR/$CEF_PACKAGE_NAME"
    fi
    if [ -f "$TMP_DIR/$CEF_DOWNLOAD_FILE" ]; then
        log_message "Removing $TMP_DIR/$CEF_DOWNLOAD_FILE"
        $RM -rf "$TMP_DIR/$CEF_DOWNLOAD_FILE"
    fi
    if [ "$attempting_copy" == "true" ]; then
        $RM -rf "$CEF_INSTALL_DIR/*"
    fi
    $RM -rf "$TMP_DOWNLOAD_DIR/*"
    $RM -rf "$TMP_DOWNLOAD_DIR"
    log_message "Cleanup done"
}

function download_cef()
{
    log_message "Downloading CEF from $CEF_URL..."

    RETRIES=3
    if [ -f "$CURL" ]; then
        log_message "Downloading using curl..."
        $CURL --retry $RETRIES -C - $CEF_URL -o "$TMP_DIR/$CEF_DOWNLOAD_FILE" --connect-timeout 30
    elif [ -f "$WGET" ]; then
        log_message "Downloading using wget..."
        $WGET --tries $RETRIES --continue --show-progress $CEF_URL --directory-prefix="$TMP_DIR" --connect-timeout=30
    else
        return_error 8 "Error: curl or wget are needed for downloading."
    fi

    if [ $? -ne 0 ]; then
        return_error 4 "Error: Download failed. Please try again."
    fi
    log_message "Download successful"
}

function validate_lib_checksum()
{
    log_message "Validating the CEF lib..."
    sha2checksum=`$SHASUM -a 256 "$CEF_CHECK_INSTALL_PATH" | $CUT -d' ' -f1`
    if [ "$sha2checksum" != "$CEF_LIB_SHA2" ]; then
        log_message "$CEF_DISPLAY_NAME latest version is not installed"
        exit 5
    fi
    log_message "CEF lib Validation successful"
}

function validate_checksum()
{
    log_message "Validating the downloaded package..."
    sha2checksum=`$SHASUM -a 256 "$TMP_DIR/$CEF_DOWNLOAD_FILE" | $CUT -d' ' -f1`
    if [ "$sha2checksum" != "$CEF_PACKAGE_SHA2" ]; then
        return_error 5 "Error: Download package file verification failed. Please try again"
    fi
    log_message "Validation successful"
}

function extract_cef()
{
    log_message "Extracting the downloaded package..."
    if [ "$PLATFORM" == "Linux" ]; then
        $TAR --checkpoint=.1000 -jxf $TMP_DIR/$CEF_DOWNLOAD_FILE -C $TMP_DIR/ $CEF_RESOURCE_DIR $CEF_RELEASE_DIR
    else
        $UNZIP "$TMP_DIR/$CEF_DOWNLOAD_FILE" -d "$TMP_DIR/"
    fi
    if [ $? -ne 0 ]; then
        return_error 6 "Error: Extraction failed. Please try again"
    fi
    log_message "Extraction successful"
}

function copy_cef()
{
    log_message "Copying the $CEF_DISPLAY_NAME files..."
    if [ -d "$CEF_INSTALL_DIR" ]; then
        $RM -rf "$CEF_INSTALL_DIR/*"
    else
        if [ "$PLATFORM" == "Linux" ]; then
            $MKDIR -p "$CEF_INSTALL_DIR"
        fi
    fi

    attempting_copy=true
    if [ "$PLATFORM" == "Linux" ]; then
        $CP -r $TMP_DIR/$CEF_RESOURCE_DIR $CEF_INSTALL_DIR/
        if [ $? -ne 0 ]; then
            return_error 7 "Error: Copying of files to $CEF_INSTALL_DIR failed"
        fi

        $CP -r $TMP_DIR/$CEF_RELEASE_DIR $CEF_INSTALL_DIR/
        if [ $? -ne 0 ]; then
            return_error 7 "Error: Copying of files to $CEF_INSTALL_DIR failed"
        fi

        $CP -r $CEF_INSTALL_DIR/Resources/* $CEF_INSTALL_DIR/Release/
        if [ $? -ne 0 ]; then
            return_error 7 "Error: Copying of files to $CEF_INSTALL_DIR failed"
        fi
    else
        $CP -rf "$TMP_DIR/${CEF_DIRNAME}" "${CEF_INSTALL_DIR2}"
        if [ $? -ne 0 ]; then
            return_error 7 "Error: Copying of files to $CEF_INSTALL_DIR failed"
        fi
    fi
    log_message "Copying successful"
}

function install()
{
    log_message "$CEF_DISPLAY_NAME"

    if [ "$REINSTALL_CEF" == "false" ]; then
        check_installed
        if [ "$ALREADY_INSTALLED" == "true" ]; then
            return_error 0 "Not installing again. Please use 'reinstall' option for reinstallation."
        fi
    fi
    check_root_privilege
    create_secure_dir
    if [ ! -d "$PULSE_LOG_DIR" ]; then
        $MKDIR -p "$PULSE_LOG_DIR"
    fi
    LOG_DIR="$PULSE_LOG_DIR"
    check_disk_space
    download_cef
    validate_checksum
    extract_cef
    copy_cef
    cleanup
    log_message "Successfully installed $CEF_DISPLAY_NAME"
}

function uninstall()
{
    check_root_privilege
    if [ -d "$CEF_INSTALL_DIR" ]; then
        $RM -rf "$CEF_INSTALL_DIR"
        if [ $? -ne 0 ]; then
            return_error 8 "Error: Removal of $CEF_INSTALL_DIR failed"
        else
    	    log_message "Successfully uninstalled $CEF_DISPLAY_NAME"
        fi
    fi
}
#
# Main
#

if [ ! -d "$LOG_DIR" ] || [ $EUID -ne 0 ]; then
    LOG_DIR=/tmp
fi
# Deleting residuals and temporary download directories 
 $ECHO "Deleting residuals and temporary download directories for CEF Browser"
 $RM -rf "$TMP_DOWNLOAD_DIR/*"
 $RM -rf "$TMP_DOWNLOAD_DIR"

if [ "$1" == "check_installed" ]; then
    check_installed
elif [ "$1" == "install" ] || [ "$1" == "reinstall" ]; then
    if [ "$2" == "-$TMP_DIR_PARAM" ]; then
        if [ "$3" != "" ]; then
            USR_TMP_DIR=$3
        fi
    elif [ "$2" != "" ]; then
        printUsage
    fi
    if [ "$1" == "reinstall" ]; then
        REINSTALL_CEF=true
    fi
    install
elif [ "$1" == "uninstall" ]; then
    uninstall
else
    printUsage
fi


